from typing import Optional

import polars as pl
from transformers import pipeline


def add_sentiment(df: pl.DataFrame, lang: str = "en") -> pl.DataFrame:
    """
    Add sentiment column to the input dataframe

    Parameters:
    df (pl.DataFrame): The input dataframe
    lang (str): Language of the messages, defaults to "en"

    Returns:
    pl.DataFrame: The input dataframe with an additional column "sentiment"

    """
    if "message" not in df.columns:
        raise ValueError("Input dataframe does not contain a 'message' column")

    model_path = (
        "cardiffnlp/twitter-roberta-base-sentiment-latest"
        if lang == "en"
        else "cardiffnlp/twitter-xlm-roberta-base-sentiment"
    )
    sentiment_pipeline = pipeline("sentiment-analysis", model=model_path)

    def extract_sentiment(message: str) -> Optional[str]:
        """
        Extract sentiment from message

        Parameters:
        message (str): The input message

        Returns:
        str: The sentiment of the message

        """
        try:
            return sentiment_pipeline(message)[0]["label"]
        except Exception as e:
            print(f"Error processing message: {message}: {e}")
            return None

    df = df.with_columns(
        (pl.col("message")).map_elements(extract_sentiment).alias("sentiment"),
    )
    return df



#!/usr/bin/env python3

import argparse
import sys

from chatminer.chatparsers import (
    FacebookMessengerParser,
    InstagramJsonParser,
    SignalParser,
    TelegramJsonParser,
    WhatsAppParser,
)


def get_args():
    try:
        cliparser = argparse.ArgumentParser(
            description="chat-miner provides lean parsers for every major platform transforming chats into dataframes.\
             Artistic visualizations allow you to explore your data and create artwork from your chats."
        )

        cliparser.add_argument(
            "-p",
            "--parser",
            type=str,
            help="The platform from which the chats are imported",
            choices=["whatsapp", "instagram", "facebook", "signal", "telegram"],
        )

        cliparser.add_argument(
            "-i", "--input", type=str, help="Input file to be processed"
        )

        cliparser.add_argument(
            "-o", "--output", type=str, help="Output file for the results"
        )

        return cliparser.parse_args(), cliparser
    except KeyboardInterrupt:
        sys.exit()


def main():
    try:
        args, cliparser = get_args()

        if args.input is None or args.output is None or args.parser is None:
            raise ValueError

        if (args.parser).lower() == "whatsapp":
            chatparser = WhatsAppParser(args.input)

        elif (args.parser).lower() == "facebook":
            chatparser = FacebookMessengerParser(args.input)
        elif (args.parser).lower() == "instagram":
            chatparser = InstagramJsonParser(args.input)
        elif (args.parser).lower() == "signal":
            chatparser = SignalParser(args.input)
        elif (args.parser).lower() == "telegram":
            chatparser = TelegramJsonParser(args.input)
        else:
            raise ValueError

        chatparser.parse_file()
        df = chatparser.parsed_messages.get_df()
        df.to_csv(args.output, index=False)

    except ValueError:
        cliparser.print_usage()

    except KeyboardInterrupt:
        sys.exit()


if __name__ == "__main__":
    main()
